module Space where

import Prelude hiding ((<*))
import Shape(Shape(Polygon))
import Region(Region)
import Picture(Picture)
import Animation(picToGraphic)
import SOE(Color)
import Fal

import Debug.Trace(trace)

-- Rotate a point by an angle around the origin
rot :: Float-> (Float, Float)-> (Float, Float)
rot w (x, y)= (x* cos w- y* sin w, x* sin w+ y* cos w)

-- Convert polar coordinates to cartesian and vice versa
cart :: Float-> Float-> (Float, Float)
cart r w = rot w (r, 0)
polar :: (Float, Float)-> (Float, Float)
polar (x, y) = (sqrt (x*x+ y*y), atan2 y x)

-- Draw a polygon
poly :: Behavior [(Float, Float)]-> Behavior Region
poly pts = shape $ lift1 Polygon pts

-- Key press events to start/stop acceleration and breaking
startAcc, stopAB, startBrk :: Event ()
startAcc = keyPress 'a'
stopAB   = keyPress 's'
startBrk = keyPress 'd'

-- Steering (directly from the mouse)
steering :: Behavior Float
steering = negate (fst mouse)

-- Paint the steering wheel
steering_wheel :: Behavior Picture
steering_wheel =  paint red $ translate (negate steering, -2.5) (rec 0.2 0.05)

-- Direction (orientation) of the car
dir :: Behavior Float
dir = integral (steering * speed)

-- Acceleration: start/stop as per key, cut-off when speed gets negative
acceleration :: Behavior Float
acceleration = 0 `untilB` ca where
   ca = (startAcc ->> 2 `untilB` ca) .|.
        (stopAB ->> 0 `untilB` ca) .|.
        (startBrk ->> (-10) `untilB` ca) -- .|.

-- Integral bounded below by zero
posIntegral :: Behavior Float-> Behavior Float
posIntegral = genIntegral 0 (\acc x-> max 0 (acc+ x)) (*)

-- Linear speed
speed :: Behavior Float
speed = posIntegral (acceleration)

-- Velocity vector
velocity :: Behavior (Float, Float)
velocity = lift2 cart speed dir

-- Position of the car
pos :: Behavior (Float, Float)
pos = vecIntegral velocity

-- Color of the car changes when braking/accelerating
car_col :: Behavior Color
car_col = blue `untilB` sa where
    sa = (startAcc ->> red `untilB` sa) .|.
         (stopAB ->> blue `untilB` sa) .|.
         (startBrk ->> green `untilB` sa)

-- Draw the car
car :: Behavior Picture
car = paint car_col $ move pos $
          poly $ lift2 map (lift1 rot dir) (constB carshape) where
   carshape = [(0.4, 0), (0, 0.1), (0, -0.1)]

main :: IO ()
main = reactimate "Foosh!" $ lift1 picToGraphic (car `over` steering_wheel)

